/*
 * calc.c
 *
 * Josef Kufner <jkufner@seznam.cz>
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>

#include "main.h"
#include "debug.h"
#include "calc.h"
#include "lists.h"
#include "convert.h"

/* Dle obsahu seznamu provede potrebne substituce */
char *subst(char *expr, LIST_ID last_level) {
	LIST_ITEM *item;
	char *p_var = NULL, *p_tmp = NULL;
	static char *p_expr = NULL;
	char var[MAX_LEN];
	int found = 0, i = 0, is_hd = 0;
	LIST_ID list[] = { VAR_LIST, ADD_LIST, MAX_LIST, MAX_LIST };

	
	PRINT_DEBUG("Substituting expression \"%s\"", expr);

	list[2] = last_level;

	if (p_expr)
		free(p_expr);
	
	p_expr = (char *) malloc_c((strlen(expr) + 1) * sizeof(char));
	strcpy(p_expr, expr);

	while ((p_var = strchr(p_expr, '$')) != NULL) {
		
		/* skip leading '$' */
		p_var += sizeof(char);
		
		/* copy variable name */
		i = 0;
		while(p_var[i] != '\0' && (isalnum(p_var[i]) || p_var[i] == '_')) {
			var[i] = p_var[i];
			i++;
			var[i] = '\0';
		}

		/* compare with known names in the lists */
		for(i = 0; list[i] != MAX_LIST; i++) {
			push_list_pos(list[i]);
			rewind_list(list[i]);
			
			while ((item = get_item(list[i])) != NULL && strcmp(item->name, var) != 0)
				;
			
			pop_list_pos(list[i]);
			
			if (item != NULL) {
				PRINT_DEBUG_PRG("i = %d", i);
				if (list[i] == HEAD_LIST) {
					PRINT_DEBUG_POINT();
					is_hd = 1;
				}
				break;
			}
		}

		if (!item) {
			PRINT_ERR_FILE("Unknown variable \"%s\"", var);
			exit(EXIT_ERROR);
		}

		/* malloc needed memory */
		p_tmp = (char *) malloc_c((strlen(p_expr) - strlen(var) + strlen(item->expr)) * sizeof(char));

		/* copy and substitute strings */
		strncpy(p_tmp, p_expr, (p_var - p_expr - 1)/sizeof(char));
		p_tmp[(p_var - p_expr - 1)/sizeof(char)] = '\0';
		strcat(p_tmp, is_hd ? convert_hd_to_var(item->expr) : item->expr);
		strcat(p_tmp, p_var + (strlen(var) * sizeof(char)));
	
		/* free old expression and redirect pointer to new expression */
		free(p_expr);
		p_expr = p_tmp;

		PRINT_DEBUG_L2("Substituting... \"%s\"", p_expr);
	}

	PRINT_DEBUG("     Result: \"%s\"", p_expr);
	
	return p_expr;
}

/* Prozene expr kalkulackou */
char *calc(char *expr, FORMAT_MODE format_mode, int format_digits) {
	static char result[MAX_LEN];
	char *p_result;
	char cmd[MAX_LEN + 128];
	FILE *f_calc;
	int ret;

	char *mode_str[] = { "real", "sci" };
	
	PRINT_DEBUG("Calculating expression \"%s\"", expr);

	/* little hack for empty cols */
	if (strcmp(expr, ".") == 0)
		return "";

	
	sprintf(cmd, "calc -p '"
			"config(\"mode\", \"%s\"); "
			"config(\"display\", %d); "
			"config(\"tilde\", 0); "
			"config(\"leadzero\", 1); "
			"config(\"fullzero\", 1); "
			"%s'",
			
			mode_str[format_mode],
			format_digits,
			expr);
		
	PRINT_DEBUG_L2("popen: %s", cmd);
		
	/* Run calc */
	if((f_calc = popen(cmd, "r")) == NULL) {
		PRINT_ERR_PRG("Fatal: Can't run \"%s\": %s", cmd, strerror(errno));
		exit(EXIT_ERROR);
	}

	/* Read last output line */
	while (!feof(f_calc)) {
		/* get one line */
		fgets(result, MAX_LEN, f_calc);

		/* replace '\n' -> '\0' */
		if ((p_result = strchr(result, '\n')) != NULL)
			*p_result = '\0';

		/*PRINT_DEBUG_L2("calc: %s", result);*/
	}
	
	ret = pclose(f_calc);
	
	PRINT_DEBUG("     Result: \"%s\"", result);
	return result;
}

