/*
 * debug.h
 *
 * Josef Kufner <jkufner@seznam.cz>
 * 
 */

#include <errno.h>

#include "main.h"

#ifdef ENABLE_DEBUG

#define PRINT_ERR_PRG(str, arg...)	{fprintf(stderr, "%s: " str "\n", prg_name, ##arg);}
#define PRINT_ERR_FILE(str, arg...)	{fprintf(stderr, "%s:%4d: " str "\n", in_file, line_no, ##arg);}

#define PRINT_DEBUG(arg...)		{if (debug) PRINT_ERR_FILE(##arg); }
#define PRINT_DEBUG_PRG(str, arg...)	{if (debug) fprintf(stderr, "%s:%s:%d: " str "\n", \
						prg_name, __FILE__, __LINE__, ##arg); }
#define PRINT_DEBUG_EMPTY_LINE()	{if (debug == 2) fprintf(stderr, "\n");}

#define PRINT_DEBUG_L2(arg...)		{if (debug == 2) PRINT_DEBUG(##arg); }
#define PRINT_DEBUG_PRG_L2(arg...)	{if (debug == 2) PRINT_DEBUG_PRG(##arg);}

#define PRINT_VERBOSE(arg...)		{if (verbose) PRINT_ERR_FILE(##arg); }
#define PRINT_VERBOSE_PRG(arg...)	{if (verbose) PRINT_ERR_PRG(##arg); }

#define PRINT_DEBUG_POINT()		{if (debug) PRINT_ERR_PRG("Reached line %d in %s.", __LINE__, __FILE__);}

#else

#define PRINT_ERR_PRG(str, arg...)
#define PRINT_ERR_FILE(str, arg...)

#define PRINT_DEBUG(arg...)
#define PRINT_DEBUG_PRG(str, arg...)
					
#define PRINT_DEBUG_EMPTY_LINE()

#define PRINT_DEBUG_L2(arg...)
#define PRINT_DEBUG_PRG_L2(arg...)

#define PRINT_VERBOSE(arg...)
#define PRINT_VERBOSE_PRG(arg...)

#define PRINT_DEBUG_POINT()

#endif
