/*
 * generator.c
 *
 * Josef Kufner <jkufner@seznam.cz>
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>

#include "main.h"
#include "debug.h"
#include "calc.h"
#include "generator.h"
#include "lists.h"
#include "convert.h"


struct {
	FILE *f_latex, *f_plain;
	char *format;
	int format_done;
} gen_info = {
	NULL, NULL,
	NULL,
	0
};

static int example_line = -1, data_line = 0, latex_table_end = 0;


/****************************************************************************/

int gen_set_format(char *format) {
	
	if (gen_info.format)
		free(gen_info.format);
	
	gen_info.format = (char *) malloc_c((strlen(format) + 1) * sizeof(char));

	strcpy(gen_info.format, format);
	
	fprintf(gen_info.f_latex, "\\begin{center}\n"
			"\\begin{tabular}{%s}\n"
			"\\hline\n", gen_info.format);

	
	return 1;
}

/* Generovani vystupu v LaTeX */
int gen_latex_begin(char *file_name) {

	if (!file_name) {
		gen_info.f_latex = stdout;
	} else {
		gen_info.f_latex = fopen(file_name, "w");
	}

	if (!gen_info.f_latex) {
		PRINT_ERR_PRG("Can't open output file \"%s\"", file_name);
		exit(EXIT_ERROR);
	}
	
	fprintf(gen_info.f_latex, "%% Generated by TeX Table from file \"%s\".\n", file_name ? file_name : "<stdin>");

	return 0;
}

int gen_latex_head(void) {
	LIST_ITEM *item;
	int col = 0;

	rewind_list(HEAD_LIST);

	while ((item = get_item(HEAD_LIST)) != NULL) {
		fprintf(gen_info.f_latex, "%s\t{\\hspace{\\stretch{1}}%s\\hspace{\\stretch{1}}}", col ? " &\n": "", item->expr);
		col++;
	}
	fprintf(gen_info.f_latex, " \\\\\n"
			"\\hline\n");

	
	
	return 0;
}

int gen_latex_data(void) {
	LIST_ITEM *item;
	char val[MAX_LEN];
	char str[256];
	char number[256];
	int col = 0;
	
	push_list_pos(DATA_LIST);
	rewind_list(DATA_LIST);

	push_list_pos(FORMAT_LIST);
	rewind_list(FORMAT_LIST);
	
	data_line++;
	
	while ((item = get_item(DATA_LIST)) != NULL) {
		
		PRINT_DEBUG_PRG("Formating number: \"%s\"", item->val);
		strcpy(val, item->val);


		sprintf(number, "\t%s$%s$", col ? "& ": "", convert_val_to_tex(val));
		
		col++;
	
		PRINT_DEBUG_PRG("Formated number: \"%s\"", number);
		fprintf(gen_info.f_latex, "%s", number);
	}
	fprintf(gen_info.f_latex, " \\\\\n");
	pop_list_pos(DATA_LIST);
	pop_list_pos(FORMAT_LIST);

	return 0;
}

int gen_latex_table_end(void) {

	fprintf(gen_info.f_latex, "\\hline\n"
			"\\end{tabular}\n"
			"\\end{center}\n");
	latex_table_end = 1;
	return 1;
}

int gen_latex_raw(char *raw_line) {

	fprintf(gen_info.f_latex, "%s\n", raw_line);
	return 1;
}


int gen_set_example_line(void) {
	
	example_line = data_line + 1;

	return 1;
}

int gen_latex_example(void) {
	LIST_ITEM *hd_item, *data_item;
	int is_first = 1;

	if (!latex_table_end)
		gen_latex_table_end();

	if (example_line == -1) {
		PRINT_ERR_FILE("%s", "Data for example are undefined ! (use EXLINE)");
		exit(EXIT_ERROR);
	}
	
	fprintf(gen_info.f_latex, "\\noindent {\\bf Pklad vpotu} (%d. dek){\\bf :}\n", example_line);
	fprintf(gen_info.f_latex, "\\begin{eqnarray}\n\\ ");
	
	push_list_pos(HEAD_LIST);
	push_list_pos(EXAMPLE_LIST);
	rewind_list(HEAD_LIST);
	rewind_list(EXAMPLE_LIST);

	while ((hd_item = get_item(HEAD_LIST)) != NULL && (data_item = get_item(EXAMPLE_LIST)) != NULL) {

		/* Tisk radku */
		fprintf(gen_info.f_latex, "%s\t%s & = & %s = dosazeni = \\dblund{%s}",
				is_first ? "" : "\\nonumber\\\\\n",
				convert_hd_to_var(hd_item->expr),
				convert_val_to_tex(subst(data_item->expr, HEAD_LIST)),
				convert_val_to_tex(data_item->val));

		/* TODO
		 *
		 * - hodnoty brat z val,
		 * - kopirovani hodnot je chybne
		 */
		
		is_first = 0;
	}
	
	pop_list_pos(HEAD_LIST);
	pop_list_pos(EXAMPLE_LIST);
	
	fprintf(gen_info.f_latex, "\n\\end{eqnarray}\n");
	return 0;
}

int gen_latex_end(void) {

	if (!latex_table_end)
		gen_latex_table_end();
	
	return fclose(gen_info.f_latex) == 0;
}


/* Generovani pouze hodnot pro GNUPlot */
int gen_plain_begin(char *file_name) {
	
	if (!file_name) {
		gen_info.f_plain = stdout;
	} else {
		gen_info.f_plain = fopen(file_name, "w");
	}

	if (!gen_info.f_plain) {
		PRINT_ERR_PRG("Can't open output file \"%s\"", file_name);
		exit(EXIT_ERROR);
	}
	
	return 0;
}

int gen_plain_data(void) {
	LIST_ITEM *item;
	int col = 0;

	push_list_pos(DATA_LIST);
	rewind_list(DATA_LIST);

	while ((item = get_item(DATA_LIST)) != NULL) {
		fprintf(gen_info.f_plain, "\t%s", item->val[0] ? item->val : "0");
		col++;
	}
	fprintf(gen_info.f_plain, "\n");
	pop_list_pos(DATA_LIST);

	return 0;
}

int gen_plain_end(void) {

	return fclose(gen_info.f_plain) == 0;
}

