/*
 * lists.c
 *
 * Josef Kufner <jkufner@seznam.cz>
 *
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "main.h"
#include "debug.h"
#include "lists.h"

#define CHECK_LIST_ID()		{if (list_id >= MAX_LIST) return 0;}

#define STACK_SIZE		5

/*
 * Promenne -- CALC
 * Automat. gen -- ADD
 * Data z jedne radky
 * 
 */

/* Return values:
 * 	1 = succes
 * 	0 = error
 */

static struct {
	LIST_ITEM *base;
	LIST_ITEM *curr;
	
	LIST_ITEM *stack[STACK_SIZE];
	unsigned stack_level;
} list[] = {
	/* VAR_LIST */		{ NULL, NULL, { NULL }, 0 },
	/* ADD_LIST */		{ NULL, NULL, { NULL }, 0 },
	/* DATA_LIST */		{ NULL, NULL, { NULL }, 0 },
	/* HEAD_LIST */		{ NULL, NULL, { NULL }, 0 },
	/* FORMAT_LIST */	{ NULL, NULL, { NULL }, 0 },
	/* EXAMPLE_LIST */	{ NULL, NULL, { NULL }, 0 }
};

char *list_name[] = { "var_list", "add_list", "data_list", "head_list", "format_list", "example_list", "(max_list)" };


int create_list(LIST_ID list_id) {

	CHECK_LIST_ID();
	PRINT_DEBUG_PRG_L2("Creating %s", list_name[list_id]);

	if (list[list_id].base) {
		PRINT_ERR_PRG("Internal Error: List %s alerady created.", list_name[list_id]);
	}

	PRINT_DEBUG_POINT();
	
	list[list_id].base = (LIST_ITEM *) malloc_c(sizeof(LIST_ITEM));
	PRINT_DEBUG_POINT();
	
	list[list_id].base->next = NULL;
	strcpy(list[list_id].base->name, "");
	strcpy(list[list_id].base->expr, "");
	strcpy(list[list_id].base->val, "");
	list[list_id].base->format_mode = NORMAL;
	list[list_id].base->format_digits = 2;
	PRINT_DEBUG_POINT();
	
	list[list_id].curr = NULL;

	return 1;
}



int destroy_list(LIST_ID list_id) {
	LIST_ITEM *next;
	
	CHECK_LIST_ID();
	PRINT_DEBUG_PRG_L2("Destroying %s", list_name[list_id]);

	if (list[list_id].stack_level) {
		PRINT_ERR_PRG("%s", "Internal Error: Stack is not empty!");
	}
	
	for (list[list_id].curr = list[list_id].base;
			list[list_id].curr != NULL;
			list[list_id].curr = next)
	{
		next = list[list_id].curr->next;
		free(list[list_id].curr);
	}

	list[list_id].base = NULL;
	list[list_id].curr = NULL;
	
	return 0;
}



int clear_list(LIST_ID list_id) {
	
	CHECK_LIST_ID();
	PRINT_DEBUG_PRG_L2("Clearing %s", list_name[list_id]);

	destroy_list(list_id);		/* nevadi pokud nejde znicit, */
	return create_list(list_id);	/* ale musi jit vytvorit */

	return 1;
}



int rewind_list(LIST_ID list_id) {
	
	CHECK_LIST_ID();
	PRINT_DEBUG_PRG_L2("Rewinding %s", list_name[list_id]);

	if (list[list_id].base) {
		list[list_id].curr = list[list_id].base->next;
		return 1;
	}
	return 0;
}



int is_list_empty(LIST_ID list_id) {
	int ret;
	
	CHECK_LIST_ID();
	ret = list[list_id].base && list[list_id].base->next ? 0 : 1;
	PRINT_DEBUG_L2("List is%sempty.", ret ? " " : " NOT ");
	return ret;
}



LIST_ITEM *create_item(LIST_ID list_id) {
	LIST_ITEM *item, *next;
	
	CHECK_LIST_ID();
	PRINT_DEBUG_PRG_L2("Creating item in %s", list_name[list_id]);

	/* Check if list exists */
	if (list[list_id].base == NULL) {
		PRINT_ERR_PRG("Internal Error: Can't create item in nonexisting list \"%s\"", list_name[list_id]);
		exit(EXIT_ERROR);
	}

	/* Create Item */
	item = (LIST_ITEM *) malloc_c(sizeof(LIST_ITEM));
	strcpy(item->expr, "");
	strcpy(item->val, "");
	strcpy(item->name, "");
	item->format_mode = NORMAL;
	item->format_digits = 2;

	/* Insert item into list */
	if (list[list_id].curr) {
		/* Item is NOT first */
		next = list[list_id].curr->next;
		list[list_id].curr->next = item;
		list[list_id].curr = item; 
		item->next = next;
	} else {
		/* Item is first */
		list[list_id].base->next = item;
		list[list_id].curr = item;
		item->next = NULL;
	}

	return list[list_id].curr;
}



LIST_ITEM *get_item(LIST_ID list_id) {
	LIST_ITEM *curr;
	
	CHECK_LIST_ID();
	PRINT_DEBUG_PRG_L2("Getting item from %s", list_name[list_id]);

	if ((curr = list[list_id].curr) != NULL) {
		list[list_id].curr = list[list_id].curr->next; 
	} else {
		PRINT_DEBUG_PRG_L2("End of %s", list_name[list_id]);
	}
	
	return curr;
}

int push_list_pos(LIST_ID list_id) {

	CHECK_LIST_ID();
	PRINT_DEBUG_PRG_L2("Pushing %s position to the Stack (level=%d)", list_name[list_id], list[list_id].stack_level);

	if (list[list_id].stack_level < STACK_SIZE) {
		list[list_id].stack_level++;
		list[list_id].stack[list[list_id].stack_level] = list[list_id].curr;
	} else {
		PRINT_ERR_PRG("Internal error: %s stack overflow.", list_name[list_id]);
		exit(EXIT_ERROR);
	}
	return 1;
}

int pop_list_pos(LIST_ID list_id) {

	CHECK_LIST_ID();
	PRINT_DEBUG_PRG_L2("Getting %s position from the Stack (level=%d).",
			list_name[list_id], list[list_id].stack_level - 1);

	if (list[list_id].stack_level >= 1) {
		list[list_id].curr = list[list_id].stack[list[list_id].stack_level];
		list[list_id].stack_level--;
	} else {
		PRINT_ERR_PRG("Internal error: %s stack underflow.", list_name[list_id]);
		exit(EXIT_ERROR);
	}
	return 1;
}

